#include "PointSearch.h"
#include "UnstructuredGrid3D.h"


PointSearch::PointSearch(Grid& grid_,int Nmax): grid(grid_)
{
  nn_idx       = new ANNidx[Nmax];
  nn_distances = new ANNdist[Nmax];
  data_pts     = annAllocPts(grid.numPoints(),3);		//Local buffer keeping all grid point coordinates. Used by the ANN engine.
  
  for(int i=0;i<grid.numPoints();++i)					//Copy grid vertices in an ANN-compatible structure
  {
    float point[3];
	grid.getPoint(i,point);
    data_pts[i][0] = point[0];
    data_pts[i][1] = point[1];
    data_pts[i][2] = point[2];
  }
    
  kdt = new ANNkd_tree(data_pts,grid.numPoints(),3);	//Create the KD-tree used by ANN to search close points
}


void PointSearch::closest(int N,const Point3d& p,vector<int>& indexes,vector<float>& distances)
{														//Search the N closest points in the grid to the point 'p'.
  ANNpoint query_pt = annAllocPt(3);
  query_pt[0] = p.x;									//1. Pass the search point to ANN
  query_pt[1] = p.y;
  query_pt[2] = p.z;
  
  indexes.resize(N);									//2. Make sure we have enough space to return N search results
  distances.resize(N);
    
  kdt->annkSearch(query_pt, N, nn_idx, nn_distances);	//3. Search the N closest points to 'p' using ANN	

  for(int i=0;i<N;++i)									//4. Pass results back to caller
  {
	  indexes[i]   = nn_idx[i];							//4.1. Index of i-th closest point
	  distances[i] = sqrt(nn_distances[i]);				//4.2. Distance to i-th closest point. Note that ANN returns
  }														//     squared distances to the search point.

  annDeallocPt(query_pt);								//5.   Done with the local data structures
}


PointSearch::~PointSearch()
{
  delete kdt;											//Delete all ANN-related data structures
  delete[] nn_distances;
  delete[] nn_idx;
  annDeallocPts(data_pts);
}



