#pragma once

#include <GLUT/glut.h>
#include "Point3d.h"
#include "Renderer.h"


class PointCloudSurface;


class PointCloudSurfaceRenderer : public Renderer
{
public:

enum DRAW_MODE				//Style of point-cloud surface drawing:	
{
   DRAW_POINTS =0,			//-use simple unshaded constant-sized points
   DRAW_VOXELS,				//-use voxel cubes (variant of above)
   DRAW_SPLATS,				//-use shaded (oriented) points (fast, quite accurate for dense clouds)
   DRAW_GEOM_BALLS,			//-use geometric spheres (slow, accurate)
   DRAW_SPRITE_BALLS		//-like above, but using shaders (very fast and accurate)
};     

						PointCloudSurfaceRenderer();
void					draw(Grid&);

float					light_diffuse;									//lighting parameters
float					light_specular;
float					specular_power;
float					light_pos[4];
int						use_mipmaps;									//use texture mipmaps for ball splatting or not
int						use_floating_textures;							//use floating-point or 24-bit-fixed-point textures for the ball height
DRAW_MODE				draw_mode;										//how to draw the point-cloud surface
bool					use_colormap;									//whether to use scalar color-mapping of the input point-cloud or not
float					splat_radius;									//size (pixels) of splat used by drawSplats()

private:

void					initGL();													
void					makeTextures(int sampling_mode=-1);				//make all splat textures using some sampling mode
void					createSplatFloat(int size,GLfloat* img,int level);
void					createSplatRGBA(int size,GLubyte* img,int level);
void					createDisk(int size);
int						getScalingMode() const;							//return texture scaling mode depending on usage of mipmaps, etc
void					drawBalls(PointCloudSurface& surface,bool use_constant_radius);			
void					drawCubes(PointCloudSurface& surface);			
void					drawSplats(PointCloudSurface& surface);
void					drawPoints(PointCloudSurface& surface);

float					ball_dt_offset;
float					splat_dt_offset;
float                   ball_size;										//ball size (in obj normalized coords) for the constant-radius ball drawing
int						sphere_faces;									//resolution of the GLUT spheres

GLuint					balltex[2];										//texture IDs for the ball splat (fixed-point and floating-point modes)
GLuint					ball_shader[3];									//shader IDs for the splat fragments
GLuint					ball_prog;										//shader program ID
int						splat_size;										//size (pow 2) of the splat texture
int						scaling_mode;									//internal texmapping param for splats
};






