#include <GLUT/glut.h>									//GLUT library
#include <iostream>
#include "PointCloudSurface.h"
#include "PointCloudSurfaceRenderer.h"
#include "UnstructuredGrid3D.h"
#include "PlyReader.h"
#include "zpr.h"


UnstructuredGrid3D*			grid			= 0;
PointCloudSurface*			cloud			= 0;
PointCloudSurfaceRenderer*	cloud_renderer  = 0;
int							draw_mode		= 0;



static void mouseclick(int button, int state, int x, int y)
{
	int keys = glutGetModifiers();						//The mouse events are interpreted as follows:	
	if (keys & GLUT_ACTIVE_CTRL)						// 
	button = GLUT_MIDDLE_BUTTON;						// -left button + move:                         rotate viewpoint
	if (keys & GLUT_ACTIVE_SHIFT)						// -middle button (or left button+Control key): zoom in / out
	button = GLUT_RIGHT_BUTTON;							// -right buttom (or left button+Shift key):    translate viewpoint

	zprMouse(button,state,x,y);							//Use the ZPR library to manipulate the viewpoint. The library sets the modelview  
														//OpenGL matrix based on the mouse events, thereby changing the current viewpoint.
}


static void mousemotion(int x, int y)
{
	zprMotion(x,y);
	glutPostRedisplay();
}


static void viewing(int width, int height)					//GLUT window-reshape callback
{
	glutReshapeWindow(width,height);  
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70,double(width)/height,0.5,7);
	glViewport(0,0,width,height);
}


static void keyboard(unsigned char ch,int,int)				//GLUT keyboard callback
{
	switch(ch)
	{
	case ' ':												//Cycle through the voxel-surface's drawing modes
		draw_mode = (++draw_mode) % 5;	
		cloud_renderer->draw_mode = (PointCloudSurfaceRenderer::DRAW_MODE)draw_mode;
		switch (cloud_renderer->draw_mode)
		{
		case PointCloudSurfaceRenderer::DRAW_POINTS: 
		  cout<<"Drawing cloud as raw points"<<endl; break;
		case PointCloudSurfaceRenderer::DRAW_VOXELS:
		  cout<<"Drawing cloud as voxels"<<endl; break;
		case PointCloudSurfaceRenderer::DRAW_SPLATS:
		  cout<<"Drawing surface as oriented points"<<endl; break;
		case PointCloudSurfaceRenderer::DRAW_GEOM_BALLS:
		  cout<<"Drawing surface as geometric balls"<<endl; break;
		case PointCloudSurfaceRenderer::DRAW_SPRITE_BALLS:
		  cout<<"Drawing surface as sprite balls"<<endl; break;
		}        
		break;
	case 'r':												//Reset the viewpoint
	case 'R':
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		zprInit(0,0,-2);
		break;	
	case 'c':												//Toggle using the colormap or not
	case 'C':												//Done only for the cloud renderer, since the mesh has no scalars
		cloud_renderer->use_colormap = !cloud_renderer->use_colormap;
		break;	
	case '+':												//Increase radius of splat used for DRAW_SPLATS rendering mode
	case '=':
	    cloud_renderer->splat_radius *= 1.1;
		break;	
	case '-':												//Decrease radius of splat used for DRAW_SPLATS rendering mode
	case '_':
	    cloud_renderer->splat_radius = max(cloud_renderer->splat_radius/1.1,1.0);
		break;	
	}
	
	glutPostRedisplay();	
}		


static void display()										//GLUT display callback
{
   glMatrixMode(GL_MODELVIEW);	
   glPushMatrix();
   glTranslatef(0,0,-2);									//Translate the volume so it's in front of the near clipping plane

   cloud_renderer->draw(*cloud);							//Draw the point cloud

   glPopMatrix();
}


int main(int argc,char* argv[])								//Main program
{
	const char* filename = (argc>1)? argv[1]:"DATA/scapula.ply";
	
	glutInit(&argc, argv);									//1. Initialize the GLUT toolkit
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH);	//2. Ask GLUT to create next windows with a RGB framebuffer and a Z-buffer too
	glutInitWindowSize(500,500);							//3. Tell GLUT how large are the windows we want to create next
	glutCreateWindow("1. Point cloud rendering");		
	zprInit(0,0,-2);

	PlyReader rdr;											//4.  Read a 3D mesh stored in a file in the PLY format
    grid = rdr.read(filename);

    grid->normalize();										//5.  Normalize the mesh in the [-1,1] cube. This makes setting the OpenGL projection easier.

	cloud = new PointCloudSurface(*grid);					//7.  Create a point-cloud from the mesh (by dumping the mesh's cell information)
	
	for(int i=0;i<cloud->numPoints();++i)					//8.  Create a scalar field on the point cloud. For illustration purposes, we use here
	{														//    simply the radius values of the points in the cloud. Try other scalar fields!
		float radius = cloud->getPointRadius(i);
		cloud->pointScalars().setC0Scalar(i,radius);
	}
	
	cloud_renderer = new PointCloudSurfaceRenderer();		//9.  Create a renderer for the point cloud
			
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouseclick);
	glutMotionFunc(mousemotion);
	glutReshapeFunc(viewing);
	glutDisplayFunc(display);
	
	glutMainLoop();

	return 0;
}




