#include "include/utils.h"
#include "include/Point3d.h"
#include "include/pca.h"
#include <math.h>


using namespace std;



inline void sub(float* res,const float* a,const float* b)
{
  res[0]=a[0]-b[0];
  res[1]=a[1]-b[1];
  res[2]=a[2]-b[2];
}


inline void cross(const float *v1, const float *v2, float *r)
{
  r[0]=v1[1]*v2[2]-v2[1]*v1[2];
  r[1]=v2[0]*v1[2]-v1[0]*v2[2];
  r[2]=v1[0]*v2[1]-v2[0]*v1[1];
}

inline float norm(const float *v) 
{ return sqrtf(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]); }

inline float dot(const float *v1, const float *v2)
{ return v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]; }

inline float normalize(float *v)
{
  register float len = 1e-7f + norm(v);
  v[0]/=len; v[1]/=len; v[2]/=len;
  return len;
}




void float2rgb(float value,float *col)	//simple color-coding routine
{
   static const float dx = 0.8f;
   
   if (value<0.0f) value=0.0f;
   if (value>1.0f) value=1.0f;
   
   value = (6.0f-2.0f*dx)*value+dx;
   col[0] = std::max(0.0f,(3.0f-fabsf(value-4.0f)-fabsf(value-5.0f))/2.0f);
   col[1] = std::max(0.0f,(4.0f-fabsf(value-2.0f)-fabsf(value-4.0f))/2.0f);
   col[2] = std::max(0.0f,(3.0f-fabsf(value-1.0f)-fabsf(value-2.0f))/2.0f);
}



float plane(const vector<Point3d>& pts,float* norm,float* u1,float* u2, const float* onorm)
{ 
    //Compute tg-plane to this neighborhood
    float p[3],m[6];   //Return plane in (norm,u1,u2) and local surf-classifier
    float *M[3],*V[3],W[3],MM0[3],MM1[3],MM2[3],VV0[3],VV1[3],VV2[3],d[3],m0,m1,m2; 
    int i,N = pts.size();
  
    M[0] = MM0; M[1] = MM1; M[2] = MM2; V[0] = VV0; V[1] = VV1; V[2] = VV2; 

    const float* pi;												
    p[0] = p[1] = p[2] = 0;											//Initialize barycenter p and 1st moment m[]
    m[0] = m[1] = m[2] = m[3] = m[4] = m[5] = 0;

    for(i=0;i<N;i++)												//For all neighborhood-points:	
    {
       pi    = pts[i];												//Get ith nb-point-coords
       p[0] += pi[0];												//Compute avg-point (barycenter)
       p[1] += pi[1];
       p[2] += pi[2];
    }
    p[0] /= N; p[1] /= N; p[2] /= N;								//Compute p = barycenter

    for(i=0;i<N;i++)    
    {
       pi    = pts[i];												//Get ith nb-point-coords
       d[0]  = pi[0]-p[0]; d[1] = pi[1]-p[1]; d[2] = pi[2]-p[2];
       m[0] += d[0]*d[0];  m[1] += d[0]*d[1]; m[2] += d[0]*d[2];	//Compute m[] += pi x pi
       m[3] += d[1]*d[1];  m[4] += d[1]*d[2]; m[5] += d[2]*d[2];
    }

    const float k = 1.0e4;											//Bias M[][] since values can be very small (O(1.0e-6))
    M[0][0] = k*m[0]; M[0][1] = k*m[1]; M[0][2] = k*m[2];			//Fill-in symmetric M[][] for eigen-computation
    M[1][0] = k*m[1]; M[1][1] = k*m[3]; M[1][2] = k*m[4];
    M[2][0] = k*m[2]; M[2][1] = k*m[4]; M[2][2] = k*m[5];
    
    JacobiN(M,3,W,V);												//Compute the eigenvalues and vectors of M[][] in W[],V[]
																	//Results are returned in decreasing eigenval-order W[0]>W[1]>W[2]
    W[0] = fabs(W[0]); W[1] = fabs(W[1]); W[2] = fabs(W[2]);

    norm[0] = VV0[2]; norm[1] = VV1[2]; norm[2] = VV2[2];			//Normal = eigenvec of smallest eigenval
	if (u1)
	{
		u1[0]   = VV0[0]; u1[1]   = VV1[0]; u1[2]   = VV2[0];		//
		u2[0]   = VV0[1]; u2[1]   = VV1[1]; u2[2]   = VV2[1];
	}	

    ::normalize(norm);												//Normalize the three vectors (norm,u1,n2)
    if (onorm && ::dot(onorm,norm)<0)								//If the caller gave an orientation-hint for the normal,
    {  norm[0] *= -1; norm[1] *= -1; norm[2] *= -1;  }				//use it (swap computed normal along hint's orientation)

	if (u1)
	{
		float vn = ::dot(u2,norm);										//Correct u2 so it's surely perp on norm
		u2[0] -= vn*norm[0]; u2[1] -= vn*norm[1]; u2[2] -= vn*norm[2];	//and normalized
		::normalize(u2);
		::cross(u2,norm,u1);											//Define u1 as norm x u2, so we have now a basis
	}
	
	float c_l = 2*(W[1]-W[2])/(W[0]+W[1]+W[2]);
	return c_l;		
}






